"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

from .data_source import DataSource
from .text_data_source import TextDataSource

__all__ = ["DataSource", "TextDataSource"]
