"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

from .parse import parse
from .parse_object import parse_object

__all__ = ["parse", "parse_object"]
